#************************************************************************************
#   File:     Init.gdb
#   Purpose:  Initialization for external DDRAM on ATSAMA5D33_EK,
#             defining GDB function 'reset'.
#             Call it in the GDB commands after connection with:
#               source Setup/Init.gdb
#               reset
#   Date:     02 April 2014
#************************************************************************************
#

#
# define 'reset' command
#
define reset

# Reset the chip to get to a known state
  monitor reset
# DDR reset
#MPDDRC->MPDDRC_LPR = MPDDRC_LPR_LPCB_DEEP_PWD |MPDDRC_LPR_CLK_FR_ENABLED
  set *0xFFFFEA1C = 0x07

# Disable DDR clock
#PMC->PMC_PCDR1 |= (1 << (ID_MPDDRC-32))
  set $temp_var = *0xFFFFFD08
  set $temp_var |= 0x00020000

# Enable MPDDR controller clock
  set *0xFFFFFD04 = $temp_var
# PMC->PMC_SCDR  |= PMC_SCER_DDRCK
  set $temp_var = *0xFFFFFC08
  set $temp_var |= 0x00000004

# System Clock Enable Register : Enable DDR clock
  set *0xFFFFFC04 = $temp_var

#
#PMC_SelectExt12M_Osc()
#
  set $REG_CKGR_MOR = 0xFFFFFC20
#(CKGR_MOR) Main Crystal Oscillator Enable
  set $CKGR_MOR_MOSCXTEN = (0x1 << 0)
#(CKGR_MOR) Main Crystal Oscillator Bypass
  set $CKGR_MOR_MOSCXTBY = (0x1 << 1)
#(CKGR_MOR) Main On-Chip RC Oscillator Enable
  set $CKGR_MOR_MOSCRCEN = (0x1 << 3)
#(CKGR_MOR) Main Oscillator Selection
  set $CKGR_MOR_MOSCSEL = (0x1 << 24)
#(PMC) Main Clock Frequency Register
  set $REG_CKGR_MCFR = 0xFFFFFC24
#(CKGR_MCFR) Main Clock Ready
  set $CKGR_MCFR_MAINFRDY = (0x1 << 16)
#(PMC) Status Register
  set $REG_PMC_SR = 0xFFFFFC68
#(PMC_SR) Main Oscillator Selection Status
  set $PMC_SR_MOSCSELS = (0x1 << 16)
#(PMC_SR) Master Clock Status
  set $PMC_SR_MCKRDY = (0x1 << 3)
  echo enable external OSC 12 MHz\n
  set $temp_var = *($REG_CKGR_MOR)
  set $temp_var |= ($CKGR_MOR_MOSCXTEN | (0x37 << 16))
  set *($REG_CKGR_MOR) = $temp_var

  echo wait Main CLK Ready \n
  while(!(*($REG_CKGR_MCFR) & $CKGR_MCFR_MAINFRDY))
  end

  #echo disable external OSC 12 MHz bypass \n
  #set $temp_var = *($REG_CKGR_MOR)
  #set $temp_var = $temp_var & ~($CKGR_MOR_MOSCXTBY)) | (0x37 << 16)
  #set *($REG_CKGR_MOR) = $temp_var

  echo switch MAIN clock to external OSC 12 MHz\n
  set $temp_var = *($REG_CKGR_MOR)
  set $temp_var |= $CKGR_MOR_MOSCSEL | (0x37 << 16)
  set *($REG_CKGR_MOR) = $temp_var

  echo wait MAIN clock status change for external OSC 12 MHz selection\n
  while(!((*($REG_PMC_SR)) & $PMC_SR_MOSCSELS))
  end

  echo In case when MCK is running on MAIN CLK \n
  while(!((*($REG_PMC_SR)) & $PMC_SR_MCKRDY))
  end

  #echo Disable internal RC 12 MHz\n
  #set $temp_var = *($REG_CKGR_MOR)
  #set $temp_var = ($temp_var & ~$CKGR_MOR_MOSCRCEN) | (0x37 << 16)
  #set *($REG_CKGR_MOR) = $temp_var
  set $info = *($REG_CKGR_MOR)
  echo -------- PMC_SelectExt12M_Osc ---------- is $info\n

#
#   PMC_SwitchMck2Main()
#
  echo PMC_SwitchMck2Main()\n
#(PMC) Master Clock Register
  set $REG_PMC_MCKR = 0xFFFFFC30
#(PMC_PCK[3]) Main Clock is selected
  set $PMC_PCK_CSS_MAIN_CLK = (0x1 << 0)
#(PMC_SR) Master Clock Status
  set $PMC_SR_MCKRDY = (0x1 << 3)
#(PMC) Status Register
  set $REG_PMC_SR = 0xFFFFFC68

# Select Main Oscillator as input clock for PCK and MCK
  set $temp_var = *($REG_PMC_MCKR)
  set $temp_var = ($temp_var & ~0x03)| $PMC_PCK_CSS_MAIN_CLK
  set *($REG_PMC_MCKR) = $temp_var
  while(!((*($REG_PMC_SR)) & $PMC_SR_MCKRDY))
  end
  set $info= *($REG_PMC_MCKR)
  echo  --------- PMC_SwitchMck2Main ----------- $info \n

#
#   PMC_SetPllA()
#
  echo PMC_SetPllA()\n
#(PMC) PLLA Register
  set $REG_CKGR_PLLAR = 0xFFFFFC28
#(PMC) PLL Charge Pump Current Register
  set $REG_PMC_PLLICPR = 0xFFFFFC80
#(PMC) Status Register
  set $REG_PMC_SR = 0xFFFFFC68
#(PMC_SR) PLLA Lock Status
  set $PMC_SR_LOCKA = (0x1 << 1)

  set *($REG_CKGR_PLLAR) = ((0x1 << 29) | (0x3F << 8) | ( 0 << 14) | (82 << 18) | 1 )
  set *($REG_PMC_PLLICPR) = 0
  while(!(*($REG_PMC_SR) & $PMC_SR_LOCKA))
  end
  set $info = *($REG_CKGR_PLLAR)
  echo  --------- PMC_SetPllA ---------------- $info\n

#
#   PMC_SetMckPllaDiv()
#
  echo PMC_SetMckPllaDiv()\n
#(PMC) Master Clock Register
  set $REG_PMC_MCKR = 0xFFFFFC30
#(PMC_MCKR) PLLA clock frequency is divided by 2.
  set $PMC_MCKR_PLLADIV2_DIV2 = (0x1 << 12)
  set $temp_var = *($REG_PMC_MCKR)
  set $temp_var |= $PMC_MCKR_PLLADIV2_DIV2
  set *($REG_PMC_MCKR) = $temp_var
  while(!((*($REG_PMC_SR)) & ($PMC_SR_MCKRDY)))
  end
  echo  --------- PMC_SetMckPllaDiv done ---------------- \n

#
#    PMC_SetMckPrescaler()
#
  echo PMC_SetMckPrescaler()\n
#(PMC) Master Clock Register
  set $REG_PMC_MCKR = 0xFFFFFC30
#(PMC_MCKR) Master/Processor Clock Prescaler
  set $PMC_MCKR_PRES_Msk = (0x7 << 4)
#(PMC_MCKR) Selected clock
  set $PMC_MCKR_PRES_CLOCK = (0x0 << 4)

# Change MCK Prescaler divider in PMC_MCKR register
  set $temp_var = *($REG_PMC_MCKR)
  set $temp_var = ($temp_var & ~($PMC_MCKR_PRES_Msk)) | $PMC_MCKR_PRES_CLOCK
  set *($REG_PMC_MCKR) = $temp_var
  while(!((*($REG_PMC_SR)) & ($PMC_SR_MCKRDY)))
  end
  set $info= *($REG_PMC_MCKR)
  echo  --------- PMC_SetMckPrescaler ----------$info\n

#
#   PMC_SetMckDivider()
#
  echo PMC_SetMckDivider()\n
#(PMC) Master Clock Register
  set $REG_PMC_MCKR = 0xFFFFFC30
#(PMC_MCKR) Master Clock Division
  set $PMC_MCKR_MDIV_Msk = (0x3 << 8)
#(PMC_MCKR) Master Clock is Prescaler Output Clock divided by 3.SysClk DDR is equal to 2 x MCK. DDRCK is equal to MCK.
  set $PMC_MCKR_MDIV_PCK_DIV3 = (0x3 << 8)

# change MCK Prescaler divider in PMC_MCKR register
  set $temp_var = *($REG_PMC_MCKR)
  set $temp_var = ($temp_var & ~($PMC_MCKR_MDIV_Msk)) | $PMC_MCKR_MDIV_PCK_DIV3
  set *($REG_PMC_MCKR) = $temp_var
  while(!((*($REG_PMC_SR)) & $PMC_SR_MCKRDY))
  end
  set $info = *($REG_PMC_MCKR)
  echo  --------- PMC_SetMckDivider -------------- REG_PMC_MCKR = $info\n

#
#    PMC_SwitchMck2Pll()
#
  echo PMC_SwitchMck2Pll()\n
#(PMC) Master Clock Register
  set $REG_PMC_MCKR = 0xFFFFFC30
#(PMC_MCKR) PLLACK/PLLADIV2 is selected
  set $PMC_MCKR_CSS_PLLA_CLK = (0x2 << 0)

  echo Select PLL as input clock for PCK and MCK \n
  set $temp_var = *($REG_PMC_MCKR)
  set $temp_var = ($temp_var & ~0x03) | $PMC_MCKR_CSS_PLLA_CLK
  set *($REG_PMC_MCKR) = $temp_var
  while(!((*($REG_PMC_SR)) & $PMC_SR_MCKRDY))
  end
  set $info= *($REG_PMC_MCKR)
  echo  --------- PMC_SwitchMck2Pll -------------- REG_PMC_MCKR = $info\n

  echo ------------ PLL set to 792 MHz, MCK set to 132 MHz ------------\n
  echo ------------ DDR2 is initialized ------------\n

  set *0xFFFFED00 = 0x00000001
  set *0x00800000 = 0x00000001

# Watchdog Disable
  set *0xFFFFFE44 = 0x00008000
#  Get the Chip ID  (AT91C_DBGU_C1R & AT91C_DBGU_C2R
  set $info = *0xFFFFEE40
  echo ---------------------------------------- Chip ID   $info\n

# ------------------ DDR Controller Registers --------------
#  0xFFFFEA00    Mode Register                       MPDDRC_MR
#  0xFFFFEA04    Refresh Timer Register              MPDDRC_RTR
#  0xFFFFEA08    Configuration Register              MPDDRC_CR
#  0xFFFFEA0C    Timing Parameter 0 Register         MPDDRC_TPR0
#  0xFFFFEA10    Timing Parameter 1 Register         MPDDRC_TPR1
#  0xFFFFEA14    Timing Parameter 2 Register         MPDDRC_TPR2
#  0xFFFFEA1C    Low-power Register                  MPDDRC_LPR
#  0xFFFFEA20    Memory Device Register              MPDDRC_MD
#  0xFFFFEA28    LPDDR2 Low-power Register           MPDDRC_LPDDR2_LPR
#  0xFFFFEA2C    LPDDR2 Calibration and MR4 Register MPDDRC_LPDDR2_CAL_MR4
#  0xFFFFEA30    LPDDR2 Timing Calibration Register  MPDDRC_LPDDR2_TIM_CAL
#  0xFFFFEA34    IO Calibration                      MPDDRC_IO_CALIBR
#  0xFFFFEA38    OCMS Register                       MPDDRC_OCMS
#  0xFFFFEA3C    OCMS KEY1 Register                  MPDDRC_OCMS_KEY1         Write-only
#  0xFFFFEA40    OCMS KEY2 Register                  MPDDRC_OCMS_KEY2         Write-only
#  0xFFFFEA74    DLL Master Offset Register          MPDDRC_DLL_MOR
#  0xFFFFEA78    DLL Slave Offset Register           MPDDRC_DLL_SOR
#  0xFFFFEA7C    DLL Master Status Register          MPDDRC_DLL_MSR           Read-only
#  0xFFFFEA80    DLL Slave 0 Status Register         MPDDRC_DLL_S0SR          Read-only
#  0xFFFFEA84    DLL Slave 1 Status Register         MPDDRC_DLL_S1SR          Read-only
#  0xFFFFEAE4    Write Protect Control Register      MPDDRC_WPCR
#  0xFFFFEAE8    Write Protect Status Register       MPDDRC_WPSR              Read-only
# --------------------------------------------------------------------------------------
# ---------------- DDR2 Timings (133MHz) -----------------
# -------- Configuration Register
#  n_row      =  14
#  n_col      =  10
#  n_bank     =  8
# -------- Refresh Timer Register
#  t_refresh  =  520
# -------- Timing 0 Register
#  t_tras     =  TRAS   6
#  t_trcd     =  TRCD   2
#  t_twr      =  TWR    2
#  t_trc      =  TRC    8
#  t_trp      =  TRP    2
#  t_trrd     =  TRRD   2
#  t_twtr     =  TWTR   2
#  t_tmrd     =  TMRD   2
# -------- Timing 1 Register
#  t_trfc     =  TRFC   26/14
#  t_txsnr    =  TXSNR  28/16
#  t_txsrd    =  TXSRD  208
#  t_txp      =  TXP    2
# -------- Timing 2 Register
#  t_txard    =  TXARD  7
#  t_tards    =  TXARDS 7
#  t_trpa     =  TRPA   3
#  t_trtp     =  TRTP   2
#  t_tfaw     =  TFAW  10
# -----------------------------------------------
  echo  --------- Configure MT47H64M16 DDR2 \n

  set *0xFFFFFE44 = 0x00008000
  echo Disable Watchdog\n
  set *0xFFFFEA1C = 0
  set *0xFFFFFD00 = 0x00020000
  echo Enable MPDDR controller clock\n
  set *0xFFFFFC00 = 0x00000004
  echo System Clock Enable Register : Enable DDR clock\n
  set *0xFFFFEA24 = 0x00000020
  echo DDRSDRC High Speed Register (MPDDRC_HS)  : hidden option -> calibration during autorefresh\n
  set *0xF0038004 = 0x00030000
  echo SFR_DDRCFG  DDR Configuration  Force DDR_DQ and DDR_DQS input buffer always on\n
  set *0xFFFFEA78 = 0x01010001
  echo MPDDRC DLL Slave Offset Register : set DLL Slave x Delay Line Offset\n
  set *0xFFFFEA74 = 0xC5011F07
  echo MPDDRC DLL Master Offset Register : DLL Master Delay Line Offset + DLL CLK90 Delay Line Offset + DLL Offset Selection\n
  set *0xFFFFEA34 = 0x00870a03
  echo MPDDRC I/O Calibration Register : set Resistor Divider + IO Calibration (delay between an IO Calibration Command and any Valid commands) \n
  set *0xFFFFEA24 = 0x00000028
  echo DDRSDRC High Speed Register : enable calibration during autorefresh\n
  set *0xF0038004 = 0x00030001
  echo SFR_DDRCFG  DDR Configuration : Force DDR_DQ and DDR_DQS input buffer always on\n
  set *0xFFFFEA78 = 0x01010001
  echo MPDDRC DLL Slave Offset Register : set DLL Slave x Delay Line Offset\n
  set *0xFFFFEA74 = 0xC5011f07
  echo MPDDRC DLL Master Offset Register : DLL Master Delay Line Offset + DLL CLK90 Delay Line Offset + DLL Offset Selection\n
  set *0xFFFFEA34 = 0x00870a03
  echo MPDDRC I/O Calibration Register : set Resistor Divider + IO Calibration (delay between an IO Calibration Command and any Valid commands) \n
  set *0xFFFFEA24 = 0x00000028
  echo DDRSDRC High Speed Register : enable calibration during autorefresh\n
  set *0xF0038004 = 0x00030001
  echo SFR_DDRCFG  DDR Configuration : Force DDR_DQ and DDR_DQS input buffer always on\n
  set *0xFFFFEA20 = 0x00000006
  echo Memory Device Register  :  32bit mode - DDR2 mode\n
  #set *0xFFFFEA08 = 0x00B0003D
  echo Configuration Register  :  row = 13, column(DDR) = 10, CAS 3, DLL reset disable, phase error correction is enabled / normal driver strength\n
  set *0xFFFFEA08 = 0x00B0004D
  echo Configuration Register  :  row = 14, column(DDR) = 10, CAS 3, DLL reset disable, phase error correction is enabled / normal driver strength\n
  set *0xFFFFEA0C = 0x22228226
  echo Timing 0 Register : tras  | trcd  | twr   | trc  | trp | trrd | twtr | tmrd\n
  set *0xFFFFEA10 = 0x02D0100E
  echo Timing 1 Register : trfc  | txsnr | txsrd | txp\n
  set *0xFFFFEA14 = 0x000A2277
  echo Timing 2 Register : txard | tards | trpa  | trtp | tfaw\n
  set *0xFFFFEA00 = 0x00000001
  echo Mode register : command  NOP --> ENABLE CLOCK output\n
  set *0x20000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  set $delay = 1000
  set $i = 0
  while $i != 13300
    set $i += 1
  end


  set *0xFFFFEA00 = 0x00000001
  echo Mode register : command  NOP --> ENABLE CLOCK output\n
  set *0x20000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  set $i = 0
  while $i != $delay
    set $i += 1
  end


  set *0xFFFFEA00 = 0x00000001
  echo Mode register : command  NOP --> ENABLE CLOCK output\n
  set *0x20000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  set $i = 0
  while $i != $delay
    set $i += 1
  end


  set *0xFFFFEA00 = 0x00000002
  echo Mode register : command  All Banks Precharge\n
  set *0x20000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  set $i = 0
  while $i != $delay
    set $i += 1
  end



  set *0xFFFFEA00 = 0x00000005
  echo Mode register : command  Extended Load Mode Register : Set EMR Ext Mode Reg EMSR2 BA0=0 BA1=1\n
  #set *0x24000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  set *0x28000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  set $i = 0
  while $i != $delay
    set $i += 1
  end


  set *0xFFFFEA00 = 0x00000005
  #echo Mode register : command  Extended Load Mode Register : Set EMR Ext Mode Reg EMSR3 BA0=1 BA1=1\n
  #set *0x26000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command \n
  set *0x2C000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  set $i = 0
  while $i != $delay
    set $i += 1
  end

  set *0xFFFFEA00 = 0x00000005
  echo Mode register : command  Extended Load Mode Register : Set EMR Ext Mode Reg EMSR1 BA0=1 BA1=0 ENABLE DLL\n

  #set *0x22000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command \n
  set *0x24000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command   \n
  set $i = 0
  while $i != $delay
    set $i += 1
  end

  set *0xFFFFEA08 = 0x00B000BD
  echo Configuration Register : Enable DLL reset\n
  set *0xFFFFEA00 = 0x00000003
  echo Mode register : command  RESET DLL\n
  set *0x20000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  set $i = 0
  while $i != $delay
    set $i += 1
  end

  set *0xFFFFEA00 = 0x00000002
  echo Mode register : command  All Banks Precharge\n
  set *0x20000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  set $i = 0
  while $i != $delay
    set $i += 1
  end

  set *0xFFFFEA00 = 0x00000004
  # Mode register : 2 * command  Auto-Refresh
  set *0x20000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command\n
  while $i != $delay
    set $i = $i + 1
  end


  set *0xFFFFEA00 = 0x00000004
  # Mode register :
  set *0x20000000 = 0x00000000
  # DDR2 memory : access memory to validate preeceeding command
  set $i = 0
  while $i != $delay
    set $i += 1
  end

  set *0xFFFFEA08 = 0x00B0003D
  # Configuration Register : disable DLL reset
  set *0xFFFFEA00 = 0x00000003
  echo Mode register :  MRS  initialize device operation (CAS latency, burst length and disable DLL reset)\n
  set *0x20000000 = 0x00000000
  # DDR2 memory : access memory to validate preeceeding command
  set $i = 0
  while $i != $delay
    set $i += 1
  end

  set *0xFFFFEA08 = 0x00B0703D
  # Configuration Register : OCD default value
  set *0xFFFFEA00 = 0x00000005
  # Mode register : EMRS1   OCD Default values
  #set *0x22000000 = 0x00000000
  # DDR2 memory : access memory to validate preeceeding command
  set *0x24000000 = 0x00000000
  # DDR2 memory : access memory to validate preeceeding command
  set $i = 0
  while $i != $delay
    set $i += 1
  end

  set *0xFFFFEA08 = 0x00B0003D
  # Configuration Register : OCD exit
  set *0xFFFFEA00 = 0x00000005
  # Mode register : EMRS1   OCD exit
  #set *0x22000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command \n
  set *0x24000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command \n
  set $i = 0
  while $i != $delay
    set $i += 1
  end

  set *0xFFFFEA00 = 0x00000000
  # Mode register : command  Normal mode
  set *0x20000000 = 0x00000000
  echo DDR2 memory : access memory to validate preeceeding command \n
  while $i != $delay
    set $i = $i + 1
  end


  set *0x20000000 = 0x00000000
  # DDR2 memory : access memory to validate preeceeding command
  set $i = 0
  while $i != $delay
    set $i += 1
  end

  set *0xFFFFEA04 = 0x0000061A
  # Refresh Timer register : 520 for 133 MHz
  echo ------------------------------- DDR2 memory init for 133 MHz ----------------------------------\n

# end of 'reset' command
end
